@@ -14,21 +14,21 @@ use time;
 use podio::{WritePodExt, LittleEndian};
 
 #[cfg(feature = "deflate")]
-use libflate;
+use flate2;
+#[cfg(feature = "deflate")]
+use flate2::write::DeflateEncoder;
 
 #[cfg(feature = "bzip2")]
 use bzip2;
 #[cfg(feature = "bzip2")]
 use bzip2::write::BzEncoder;
-#[allow(unused_imports)] // Rust <1.23 compat
-#[allow(deprecated)] use std::ascii::AsciiExt;
 
 enum GenericZipWriter<W: Write + io::Seek>
 {
     Closed,
     Storer(W),
     #[cfg(feature = "deflate")]
-    Deflater(libflate::deflate::Encoder<W>),
+    Deflater(DeflateEncoder<W>),
     #[cfg(feature = "bzip2")]
     Bzip2(BzEncoder<W>),
 }
@@ -123,6 +123,12 @@ impl FileOptions {
     }
 }
 
+impl Default for FileOptions {
+    fn default() -> Self {
+        Self::default()
+    }
+}
+
 impl<W: Write+io::Seek> Write for ZipWriter<W>
 {
     fn write(&mut self, buf: &[u8]) -> io::Result<usize>
@@ -260,6 +266,14 @@ impl<W: Write+io::Seek> ZipWriter<W>
         Ok(())
     }
 
+    /// Starts a file, taking a Path as argument.
+    ///
+    /// This function ensures that the '/' path seperator is used. It also ignores all non 'Normal'
+    /// Components, such as a starting '/' or '..' and '.'.
+    pub fn start_file_from_path(&mut self, path: &std::path::Path, options: FileOptions) -> ZipResult<()> {
+        self.start_file(path_to_string(path), options)
+    }
+
     /// Add a directory entry.
     ///
     /// You can't write data to the file afterwards.
@@ -284,6 +298,14 @@ impl<W: Write+io::Seek> ZipWriter<W>
         Ok(())
     }
 
+    /// Add a directory entry, taking a Path as argument.
+    ///
+    /// This function ensures that the '/' path seperator is used. It also ignores all non 'Normal'
+    /// Components, such as a starting '/' or '..' and '.'.
+    pub fn add_directory_from_path(&mut self, path: &std::path::Path, options: FileOptions) -> ZipResult<()> {
+        self.add_directory(path_to_string(path.into()), options)
+    }
+
     /// Finish the last file and write all other zip-structures
     ///
     /// This will return the writer, but one should normally not append any data to the end of the file.
@@ -354,7 +376,7 @@ impl<W: Write+io::Seek> GenericZipWriter<W>
         {
             GenericZipWriter::Storer(w) => w,
             #[cfg(feature = "deflate")]
-            GenericZipWriter::Deflater(w) =>  w.finish().into_result()?,
+            GenericZipWriter::Deflater(w) => w.finish()?,
             #[cfg(feature = "bzip2")]
             GenericZipWriter::Bzip2(w) => w.finish()?,
             GenericZipWriter::Closed => Err(io::Error::new(io::ErrorKind::BrokenPipe, "ZipWriter was already closed"))?,
@@ -364,7 +386,7 @@ impl<W: Write+io::Seek> GenericZipWriter<W>
         {
             CompressionMethod::Stored => GenericZipWriter::Storer(bare),
             #[cfg(feature = "deflate")]
-            CompressionMethod::Deflated => GenericZipWriter::Deflater(libflate::deflate::Encoder::new(bare)),
+            CompressionMethod::Deflated => GenericZipWriter::Deflater(DeflateEncoder::new(bare, flate2::Compression::default())),
             #[cfg(feature = "bzip2")]
             CompressionMethod::Bzip2 => GenericZipWriter::Bzip2(BzEncoder::new(bare, bzip2::Compression::Default)),
             CompressionMethod::Unsupported(..) => return Err(ZipError::UnsupportedArchive("Unsupported compression")),
@@ -521,6 +543,22 @@ fn build_extra_field(_file: &ZipFileData) -> ZipResult<Vec<u8>>
     Ok(writer)
 }
 
+fn path_to_string(path: &std::path::Path) -> String {
+    let mut path_str = String::new();
+    for component in path.components() {
+        match component {
+            std::path::Component::Normal(os_str) => {
+                if path_str.len() != 0 {
+                    path_str.push('/');
+                }
+                path_str.push_str(&*os_str.to_string_lossy());
+            }
+            _ => (),
+        }
+    }
+    path_str
+}
+
 #[cfg(test)]
 mod test {
     use std::io;
@@ -570,4 +608,17 @@ mod test {
         v.extend_from_slice(include_bytes!("../tests/data/mimetype.zip"));
         assert_eq!(result.get_ref(), &v);
     }
+
+    #[test]
+    fn path_to_string() {
+        let mut path = std::path::PathBuf::new();
+        #[cfg(windows)] path.push(r"C:\");
+        #[cfg(unix)] path.push("/");
+        path.push("windows");
+        path.push("..");
+        path.push(".");
+        path.push("system32");
+        let path_str = super::path_to_string(&path);
+        assert_eq!(path_str, "windows/system32");
+    }
 }
